(*
    Ocaml XML-RPC support.
    Copyright (C) 2002 Shawn Wagner <raevnos@pennmush.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*)
open Netencoding;
open Http_client;
open XmlRPCTypes;
open Pxp_yacc;
open Pxp_document;
open Pxp_types;
open Format;

exception Request_failed of string;
exception Request_fault of XmlRPCTypes.xrs;

value xml_version = "<?xml version=\"1.0\"?>";
value client_version = "0.0.1";

value agent="Ocaml-XML-RPC/" ^ client_version ^ " (" ^ Sys.os_type ^ ")";

value format_params buf args = format_one args 
  where rec format_one ls =
    match ls with
      [ [car :: cdr] -> let _ = do {
	  Buffer.add_string buf "<param>";
	  Buffer.add_string buf (XmlRPCTypes.print_type car);
	  Buffer.add_string buf "</param>"} in
	  format_one cdr
      | [] -> ()
      ] ;

value format_request b f args = 
  let buf = Buffer.create 512 in
  let _ = if (b) then do {
    Buffer.add_string buf 
      (xml_version ^ "<methodCall><methodName>" ^ f ^ "</methodName>");
    Buffer.add_string buf "<params>";
    format_params buf args;
    Buffer.add_string buf "</params></methodCall>"} 
    else do {
             Buffer.add_string buf
                   (xml_version ^ "\n<methodRem>\n<methodName>" ^ f ^"</methodName>\n");
             Buffer.add_string buf "</methodRem>\n"}
                   
    in
    Buffer.contents buf;
      
value parse_params ls = match ls with 
  [ [p::[]] -> XmlRPCTypes.parse_value p
  | _ -> raise (Request_failed "Malformed response")
  ];

value transform_dtd d = XmlRPCDtd.parsed_dtd;

(* Warning here is okay *)
value nodename n = match n#node_type with
  [ T_element name -> name
  | _ -> "D"
  ];

value handle_response xml =
  try
    let methresp =
      parse_document_entity ~transform_dtd default_config
	(from_string xml) default_spec in
    let repval = List.hd methresp#root#sub_nodes in
    let repname =nodename repval in
    let body = repval#sub_nodes in
      if repname = "fault" then
	let fault_struct = XmlRPCTypes.parse_value (List.hd body) in
	raise (Request_fault (XmlRPCTypes.ml_of_struct fault_struct))
      else (* params *)
	parse_params (List.hd body)#sub_nodes
  with
    [ Validation_error e ->
	raise (Request_failed ("Malformed response: " ^ e))
    | WF_error e ->
	raise (Request_failed ("Malformed response: " ^ e))
    | At (wh, e) -> 
	let msg = match e with
	  [ Validation_error e -> e
          | WF_error e -> e
	  | _ -> "PXP error" 
          ] in
	raise (Request_failed ("Malformed response: " ^ msg))
    ];
    
value handler = new pipeline;
handler#set_proxy_from_environment();
value options = handler#get_options ;

value _ = 
        handler#set_options { (options) with
                verbose_connection = True ;
                verbose_request_header = True ;
                verbose_request_contents = True;
                verbose_response_header = True ;
                verbose_response_contents = True;
                verbose_status = True 
        };

value make_request m trials = 
  do {
  handler#add m;
  (*XmlRPCNet.handler#add m;*)
  let rec next_trial todo e =
    if todo > 0 then 
      try
	handler#run()
        (*XmlRPCNet.handler#run()*)
      with
      [ Http_client.Http_error(n,s) as e' -> 
	    if List.mem n [408; 413; 500; 502; 503; 504 ] then
	      next_trial (todo-1) e'
	    else
	      raise e'
	| e' -> next_trial (todo-1) e'
      ]
    else
      raise e
  in
    next_trial trials (Failure "bad number of http_trials")
    } ;

value do_request b server f args =
  let methodcall = format_request b f args in
  let req = new Http_client.post_raw server methodcall in
  let _ = req#set_req_header "Connection: close" in
  let _ = req#set_req_header "Content-Type" "text/xml" in
  let _ = req#set_req_header "User-Agent" agent in
  let _ = req#set_req_header "Content-length" 
      (string_of_int (String.length methodcall)) in
  let _ =  make_request req 5 in
    if not req#is_served then 
      raise (Request_failed "Couldn't get valid response from server")
    else
      handle_response (req#get_resp_body());

class remote host func =
object
  method zero_call () = do_request True host func [];
  method simple_call arg = do_request True host func [arg];
  method call = do_request True host func;
  method remove () = do_request False host func [];
end;
    
value  r = new remote "http://quincy.inria.fr:2816" "double" ;
r#simple_call (xr_of_int 41);
