(* Copyright (c) 2002 INRIA *)

(* ----------------------------------------------------
This file gives a description of the various datatypes
and signatures used in the implementation of the server 
 ------------------------------------------------------
*)

open SoapTypes;
open SoapStructure;
open Pxp_yacc;
open Pxp_document;
open Pxp_types;

type method_name = [ Meth_Name of string ];
type auth = [Auth of bool ];

type table_entry = [Entry of method_name and (list ty -> ty)];


module type SERVER = 
sig
     value table : ref (list table_entry) ;                          
     (* table can be of any structure depending on server implementation *)  
     
     value add_handler : method_name -> (list ty -> ty) -> unit ; 
     value remove_handler : method_name -> unit ;
     value execute : method_name -> list ty (*-> auth*) -> ty ;
     exception Handler_Already_Present;
     exception Handler_Not_Present;
end ;




(* --------------------------------------------------
This file contains the interface of the XMLRPC server 
with the outside world.
-----------------------------------------------------
*)


module Server : SERVER = 
struct
     	 value table = ref [] ;
     	 exception Handler_Already_Present;
      	 exception Handler_Not_Present;
                
         value rec is_present m ls =
	     	let iscompat str e = match e with [ (Entry (Meth_Name st) g) -> (st=str)] in
		 
	     		match (m,ls) with
	 	[ (Meth_Name str, []) -> False
	 	| (Meth_Name str, [x::l]) -> or (iscompat str x)  (is_present m l)
	 	];
       
       	value add_handler s f = 
        do 
	 {
	    	let pr = is_present s (\!(table)) in
	    
	   	if (pr) then raise Handler_Already_Present
	    	else do { \:= table ((\!(table)) @ [Entry s f])} 
	 };

       	value remove_handler m = match m with 
       	[ Meth_Name s -> 
          	let pr = is_present (Meth_Name s) (\!(table)) in
	  	if (not pr) then raise Handler_Not_Present
	  	else do {\:= table (rm s (\!(table)))} 
	     	where
	         	rec rm str ls = match ls with
		 	[ [] -> []
		 	| [(Entry (Meth_Name st) g)::l] -> if (str=st) then l
		           	                            else 
							          [(Entry (Meth_Name st) g) ::  rm str l]
		 	]
         ];

       	value rec find m ls = match (m,ls) with
        [ (n,[]) -> raise Handler_Not_Present
	| (Meth_Name str, [x::l]) -> match x with
	                             [ Entry (Meth_Name st) g -> if (str=st) then g else find m l ]
	];

        value getname m = match m with [Meth_Name n -> n];

       	value execute m ls  =
                let pr = ref True in
             	let g = try find m (\!(table)) 
                                with 
                                [Handler_Not_Present -> let _ = do { \:= pr False } in
                                                        (fun h -> Unit)
                                ]
                                in 
                       if (\!(pr)) then (g ls)
                else raise Handler_Not_Present ;

       value double ls = 
                       let plist = ls in
                       let p = ml_of_decimal (List.hd plist) in
                       let p = p#get_val() in
                       let p = (2.0 *. p) in
                       let d = new Decimal_s.decimal_s in
                       let str = String.concat "" ["<def>";string_of_float p;"</def>"] in
                       let _ = d#set_obj (str) in
                       let p = sp_of_decimal d in
                                p ;

       add_handler (Meth_Name "double") double ;
end ;

module Webserver = 
    functor (Sr : SERVER) ->
        struct
                

        exception Read_Error ; 
        exception Write_Error ; 
        
        value addn = "<!ELEMENT methodRem
                                (methodName)>";

        value nodename n = match n#node_type with 
        [ T_element name -> name 
        | _ -> " D "
        ];

        value mapday n = match n with
        [ 0 -> "Sun"
        | 1 -> "Mon"
        | 2 -> "Tue"
        | 3 -> "Wed"
        | 4 -> "Thu"
        | 5 -> "Fri"
        | 6 -> "Sat"
        | _ -> "Err"
        ];

        value mapmon n = match n with 
        [ 0 -> "Jan"
        | 1 -> "Feb"
        | 2 -> "Mar"
        | 3 -> "Apr"
        | 4 -> "May"
        | 5 -> "Jun"
        | 6 -> "Jul"
        | 7 -> "Aug"
        | 8 -> "Sep"
        | 9 -> "Oct"
        | 10 -> "Nov"
        | 11 -> "Dec"
        | _ -> "Err"
        ];

        value prop s = if (String.length s > 1) then s
                       else (String.concat "" ["0";s]) ;

	value msg_body par =
           let s1 = "<Envelope><Body><methResp>" in
           let s2 = "</methResp></Body></Envelope>" in
           let s3 = print_value par in
           String.concat "" [s1;s3;s2] ;

	value make_msg par  = 
	   let body = msg_body par in
	   let s1 = "HTTP/1.1 200 OK\nConnection: close\nContent-Length:" in
	   let s3 = "\nContent-Type: text/xml" in
           let tt = Unix.gmtime (Unix.time()) in
           let t1 = mapday tt.Unix.tm_wday in
           let t2 = string_of_int tt.Unix.tm_mday in
           let t3 = mapmon tt.Unix.tm_mon in
           let t4 = string_of_int (1900 + tt.Unix.tm_year) in
           let t5 = prop(string_of_int (tt.Unix.tm_hour)) in
           let t6 = prop(string_of_int (tt.Unix.tm_min)) in
           let t7 = prop(string_of_int (tt.Unix.tm_sec)) in
           let t = String.concat "" [t1;", ";t2;" ";t3;" ";t4;" ";t5;":";t6;":";t7;" GMT\n"] in
	   let s4 = String.concat "" ["\nDate: ";t] in    
           let s44 = Sys.os_type in
           let s5 = String.concat "" ["Server: ";s44;"\n\n"] in
	   (*let s5 = "Server: UserLand Frontier/5.1.2-WinNT\n" in*)
	   let s2 = string_of_int (String.length body) in
	   let s = String.concat "" [s1;s2;s3;s4;s5;body] in
	   s ;
	
	value rec parse_params ls = match ls with
	   [ [] -> []
	   | [x :: l] -> match (x#node_type) with 
                        [ T_data -> parse_params l
                        (*| y ->  [(parse_value (List.hd (List.tl x#sub_nodes ))) :: parse_params l]*)
                        | y ->  [(parse_value (List.hd x#sub_nodes )) :: parse_params l]
                        ]
	   ];

        value make_proper msg = 
           let n = String.length msg in
           let start = String.index msg '<' in 
           let str = String.sub msg start (n-start) in
           str ;

        value rec print_list ls = match ls with
          [ [] -> ()
          | [x::l] -> do {print_string (nodename x) ; flush stdout ;print_list l}
          ];

       (* value get_fault str i =
                 let fault_code = make_struct() in
                 let _ = add_elem fault_code "faultString" (String str) in 
                 let _ = add_elem fault_code "faultCode" (sp_of_int i) in
                 let fault_code = sp_of_struct fault_code in
                 fault_code ;*)
	
	value handle_msg msg =
           (*try*)
           let msg = make_proper msg in
           let methreq = get_obj_struct (SoapXS.request) in
           let methreq = get_obj methreq msg in
           let b = methreq#get_body() in
           let ent = b#get_entrylist() in
           let repname = ml_of_string ((List.hd ent)#getval()) in
           let repname = repname#get_val() in
           let bit = ref True in
           let p = (List.hd (List.tl(ent)))#getval() in

	   let p = Sr.execute (Meth_Name repname) [p] in
	   (*let _ = try Sr.execute (Meth_Name repname) [p] 
                   with 
                  [Sr.Handler_Not_Present -> 
                         let _ = do { \:= bit False } in (get_fault "Handler Not Present" 1 ) 
                  |Bad_type -> 
                         let _ = do { \:= bit False } in (get_fault "Bad Type" 2 ) 
                  ] in*)
	   let str = make_msg p in
	               str 
          (* with
           [ Validation_error e -> let m = String.concat "" ["Malformed request: ";e] in
                                   (make_msg False (get_fault m 3))
           | WF_error e -> let m = String.concat "" ["Malformed request: ";e] in
                                   (make_msg False (get_fault m 3))
           | At (wh,e) -> let m = match e with
                             [ Validation_error e -> e
                             | WF_error e -> e
                             | _ -> "PXP error" 
                             ] in
                             let m = String.concat "" ["Malformed request: ";m] in
                                   (make_msg False (get_fault m 3))
           ]*);
				      
	value rec explode s =
 	let n = String.length s in g 0 (n-1) where
 	  rec g m h = match (m,h) with
 	     [ (k,l) -> if (k=l) then [s.[l]] else [s.[k] :: g (k+1) l]
 	     ];

        (*-----------------------------------------------------*)
        (*Error Handling in Function Call Evaluation To Be Done*)
        (*-----------------------------------------------------*)

        value mkproper_size str = 
                let pos = String.index str ' ' in
                let str = String.sub str 0 pos in
                str ;

        value andi b1 b2 = not (or (not b1) (not b2)) ;

        value handle_request s = 
                                let ch = ref 'a' in
                                let st = " " in
                                let str = ref "" in
                                let count = ref 0 in
                                let _ = do {print_string "\nReceived String: \n\n"; flush stdout} in
                                let _ = do {
                                           while ((\!(count)) < 3) do 
                                           {
                                                   let err = Unix.read s st 0 1 in
                                                     if (err < 1) then (raise Read_Error) else () ;
                                                   print_string st; flush stdout;
                                                   if (st = "\n") then do {\:= count ((\!(count)) + 1) } else ()
                                           };
                                           \:= count 0 ;
                                           while ((\!(count)) < 16) do 
                                           {
                                                   let err = Unix.read s st 0 1 in
                                                     if (err < 1) then (raise Read_Error) 
                                                      else do {\:= count ((\!(count)) + 1) };
                                                   print_string st; flush stdout
                                           };
                                           let size = ref "" in
                                           while ((\!(count)) > 0) do
                                           {
                                                   let err = Unix.read s st 0 1 in
                                                     if (err < 1) then (raise Read_Error) else () ;
                                                   print_string st; flush stdout;
                                                   if (st = "\n") then do {\:= count 0}  
                                                    else do {\:= size (String.concat "" [(\!(size));st]) }
                                           };
                                           while ((\!(count)) = 0) do
                                           {
                                                   let err = Unix.read s st 0 1 in
                                                     if (err < 1) then (raise Read_Error) else () ;
                                                   print_string st; flush stdout;
                                                   if (st = "\n") then do {\:= count 1} else () 
                                           };
                                           print_int (String.length (\!(size)));
                                           \:= size (String.sub (\!(size)) 0 (String.length (\!(size))-1));
                                           print_string (\!(size)); flush stdout;
                                           \:= count ((int_of_string (\!(size))) + 1);
                                           while ((\!(count)) >= 0) do
                                           {
                                                   let err = Unix.read s st 0 1 in
                                                      if (err < 1) then (raise Read_Error) else () ;
                                                   print_string st; flush stdout;
                                                   let _ = do {\:= count ((\!(count)) -1 ) } in
                                                    do 
                                                           {\:= str (String.concat "" [(\!(str));st])}
                                           }
                                         } in
                                         
		   		let res = handle_msg (\!(str)) in 
                                let count = ref (String.length res) in
                                let res = ref (explode res) in
                                let _ = do {print_string "\n\nMessage sent back: \n\n";flush stdout} in
		   		let _ = do
                                        {
                                                while ((\!(count)) > 0) do
                                                {
                                                        let _ = do {print_char (List.hd (\!(res)));flush stdout} in
                                                        let err = Unix.write s (String.make 1 (List.hd (\!(res)))) 0 1 in
                                                        let _ = if (err < 1) then raise Write_Error else do { \:= res (List.tl (\!(res))) } in
                                                        let _ = do { \:= count ((\!(count)) -1) } in
                                                        ()
                                                }
                                        } in
                                let _ = do {print_string "\n\n";flush stdout} in
		   		do { Unix.shutdown s Unix.SHUTDOWN_SEND}
                            ;
	 
	value establish_connection s_addr p = 
  		let inet_addr = Unix.inet_addr_of_string s_addr in
  		let sock_addr = Unix.ADDR_INET inet_addr (int_of_string p) in

  		let domain = match sock_addr with
   		[ Unix.ADDR_UNIX _ -> Unix.PF_UNIX
   		| Unix.ADDR_INET _ _ -> Unix.PF_INET
   		] in
  		let sock = Unix.socket domain Unix.SOCK_STREAM 0 in
  		let _ = Unix.bind sock sock_addr in 
  		let _ = Unix.listen sock 5 in
  		while(True) do
  		{ 
      			let (s,caller) = Unix.accept sock in
                        let t = try Thread.create handle_request s
                                with
                                [Read_Error -> let _ = do {print_string "Read Error" ; flush stdout} in Thread.self ()
                                |Write_Error -> let _ = do {print_string "Write Error" ; flush stdout} in Thread.self()]
                             in ()
  		};
end ;                   

module Wserver = Webserver(Server) ;

Wserver.establish_connection "128.93.8.52" "2815";
