(* Copyright (c) 2002 INRIA *)

(** This module contains the basic definition of various types that have been
implemented. Apart from definitions, some other functions are also defined which
enable interoperability between OCAML and SOAP-Types. *)

open String_s ;
open Boolean_s ;
open Decimal_s ;
open Float_s ; 
open Double_s ;
open Duration_s ;
open DateTime_s ;
open Time_s ;
open Date_s ;
open GYearMonth_s ;
open GYear_s ;
open GMonthDay_s ;
open GDay_s ;
open GMonth_s ;
open HexBinary_s ;
open Base64Binary_s ;
open Uri_s ;
open Qname_s ;
open Notation_s ;

(** This type contains the {i Primitive Datatypes} as defined in {b XML Schema
Part2: Datatypes} specification. Moreover, [array] and [struct] types are also
defined *)
type soapst = Hashtbl.t string ty
and ty = [ Unit 
         | String of string_s 
         | Boolean of boolean_s 
         | Decimal of decimal_s 
         | Float of float_s 
         | Double of double_s 
         | Duration of duration_s 
         | DateTime of dateTime_s 
         | Time of time_s 
         | Date of date_s 
         | GYearMonth of gYearMonth_s 
         | GYear of gYear_s 
         | GMonthDay of gMonthDay_s 
         | GDay of gDay_s 
         | GMonth of gMonth_s 
         | HexBinary of hexBinary_s 
         | Base64Binary of base64Binary_s 
         | Uri of uri_s 
         | Qname of qname_s 
         | Notation of notation_s 
         | Array of array ty
         | Struct of Hashtbl.t string ty
         ];

(** This exception is thrown when there has been a type error *)
exception Bad_type ;

(** This creates a new [soapst] struct *)
value make_struct : unit -> soapst ;

(** Insert a new key/data pair into the struct *)
value add_elem : soapst -> string -> ty -> unit ;

(** Return the data for a key *)
value find_elem : soapst -> string -> ty ;



(* Convert ML Types to SOAP - types *)

(** Converts a ML string, represented as a [string_s] object into a SOAP object *)
value sp_of_string : string_s -> ty ;

(** Converts a ML bool, represented as a [boolean_s] object into a SOAP object *)
value sp_of_boolean : boolean_s -> ty ;

(** Converts a ML decimal, represented as a [decimal_s] object into a SOAP object *)
value sp_of_decimal : decimal_s -> ty ;

(** Converts a ML float, represented as a [float_s] object into a SOAP object *)
value sp_of_float : float_s -> ty ;

(** Converts a ML double, represented as a [double_s] object into a SOAP object *)
value sp_of_double : double_s -> ty ;

(** Converts a ML duration, represented as a [duration_s] object into a SOAP object *)
value sp_of_duration : duration_s -> ty ;

(** Converts a ML dateTime, represented as a [dateTime_s] object into a SOAP object *)
value sp_of_dateTime : dateTime_s -> ty ; 

(** Converts a ML time, represented as a [time_s] object into a SOAP object *)
value sp_of_time : time_s -> ty ;

(** Converts a ML date, represented as a [date_s] object into a SOAP object *)
value sp_of_date : date_s -> ty ;

(** Converts a ML gYearMonth, represented as a [gYearMonth_s] object into a SOAP object *)
value sp_of_gYearMonth : gYearMonth_s -> ty ;

(** Converts a ML gYear, represented as a [gYear_s] object into a SOAP object *)
value sp_of_gYear : gYear_s -> ty ;

(** Converts a ML gMonthDay, represented as a [gMonthDay_s] object into a SOAP object *)
value sp_of_gMonthDay : gMonthDay_s -> ty ;

(** Converts a ML gDay, represented as a [gDay_s] object into a SOAP object *)
value sp_of_gDay : gDay_s -> ty ;

(** Converts a ML gMonth, represented as a [gMonth_s] object into a SOAP object *)
value sp_of_gMonth : gMonth_s -> ty ;

(** Converts a ML hexBinary, represented as a [hexBinary_s] object into a SOAP object *)
value sp_of_hexBinary : hexBinary_s -> ty ;

(** Converts a ML base64Binary, represented as a [base64Binary_s] object into a SOAP object *)
value sp_of_base64Binary : base64Binary_s -> ty ;

(** Converts a ML uri, represented as a [uri_s] object into a SOAP object *)
value sp_of_uri : uri_s -> ty ;

(** Converts a ML qname, represented as a [qname_s] object into a SOAP object *)
value sp_of_qname : qname_s -> ty ;

(** Converts a ML notation, represented as a [notation_s] object into a SOAP object *)
value sp_of_notation : notation_s -> ty ;

(** Converts a ML array, into a SOAP object *)
value sp_of_array : array ty -> ty ;

(** Converts a ML struct, represented as [soapst], into a SOAP object *)
value sp_of_struct : soapst -> ty ;

(** SOAP representation of UNIT *)
value skip : ty ;




(* Convert SOAP Types to ML Types *)

(** Converts a SOAP string into a ML string value *)
value ml_of_string : ty -> string_s ;

(** Converts a SOAP boolean into a ML boolean value *)
value ml_of_boolean : ty -> boolean_s ;

(** Converts a SOAP decimal into a ML decimal value *)
value ml_of_decimal : ty -> decimal_s ;

(** Converts a SOAP float into a ML float value *)
value ml_of_float : ty -> float_s ;

(** Converts a SOAP double into a ML double value *)
value ml_of_double : ty -> double_s ;

(** Converts a SOAP duration into a ML duration value *)
value ml_of_duration : ty -> duration_s ;

(** Converts a SOAP dateTime into a ML dateTime value *)
value ml_of_dateTime : ty -> dateTime_s ;

(** Converts a SOAP time into a ML time value *)
value ml_of_time : ty -> time_s ;

(** Converts a SOAP date into a ML date value *)
value ml_of_date : ty -> date_s ;

(** Converts a SOAP gYearMonth into a ML gYearMonth value *)
value ml_of_gYearMonth : ty -> gYearMonth_s ;

(** Converts a SOAP gYear into a ML gYear value *)
value ml_of_gYear : ty -> gYear_s ;

(** Converts a SOAP gMonthDay into a ML gMonthDay value *)
value ml_of_gMonthDay : ty -> gMonthDay_s ;

(** Converts a SOAP gDay into a ML gDay value *)
value ml_of_gDay : ty -> gDay_s ;

(** Converts a SOAP gMonth into a ML gMonth value *)
value ml_of_gMonth : ty -> gMonth_s ;

(** Converts a SOAP hexBinary into a ML hexBinary value *)
value ml_of_hexBinary : ty -> hexBinary_s ;

(** Converts a SOAP base64Binary into a ML base64Binary value *)
value ml_of_base64Binary : ty -> base64Binary_s ;

(** Converts a SOAP uri into a ML uri value *)
value ml_of_uri : ty -> uri_s ;

(** Converts a SOAP qname into a ML qname value *)
value ml_of_qname : ty -> qname_s ;

(** Converts a SOAP notation into a ML notation value *)
value ml_of_notation : ty -> notation_s ;

(** Converts a SOAP array into a ML array value *)
value ml_of_array : ty -> array ty ;

(** Converts a SOAP struct into a ML struct value *)
value ml_of_struct : ty -> soapst ;

(** Converts a SOAP string array into a ML string array value *)
value ml_of_stringarray : ty -> array string_s ;

(** Converts a SOAP boolean array into a ML boolean array value *)
value ml_of_booleanarray : ty -> array boolean_s ;

(** Converts a SOAP decimal array into a ML decimal array value *)
value ml_of_decimalarray : ty -> array decimal_s ;

(** Converts a SOAP float array into a ML float array value *)
value ml_of_floatarray : ty -> array float_s ;

(** Converts a SOAP double array into a ML double array value *)
value ml_of_doublearray : ty -> array double_s ;

(** Converts a SOAP duration array into a ML duration array value *)
value ml_of_durationarray : ty -> array duration_s ;

(** Converts a SOAP dateTime array into a ML dateTime array value *)
value ml_of_dateTimearray : ty -> array dateTime_s ;

(** Converts a SOAP time array into a ML time array value *)
value ml_of_timearray : ty -> array time_s ;

(** Converts a SOAP date array into a ML date array value *)
value ml_of_datearray : ty -> array date_s ;

(** Converts a SOAP gYearMonth array into a ML gYearMonth array value *)
value ml_of_gYearMontharray : ty -> array gYearMonth_s ;

(** Converts a SOAP gYear array into a ML gYear array value *)
value ml_of_gYeararray : ty -> array gYear_s ;

(** Converts a SOAP gMonthDay array into a ML gMonthDay array value *)
value ml_of_gMonthDayarray : ty -> array gMonthDay_s ;

(** Converts a SOAP gDay into a ML gDay value *)
value ml_of_gDayarray : ty -> array gDay_s ;

(** Converts a SOAP gMonth into a ML gMonth value *)
value ml_of_gMontharray : ty -> array gMonth_s ;

(** Converts a SOAP struct array into a ML struct array value *)
value ml_of_structarray : ty -> array soapst ;

(** This function takes as argument, a schema, a Pxp document node (containing
the input whose SOAP value is to be determined, and a string which contains its
type description. The resultant SOAP value is returned *)
value parse_value : Pxp_document.node (Pxp_document.extension (Pxp_document.node 'a)) -> Pxp_document.node (Pxp_document.extension (Pxp_document.node 'a)) -> string -> ty ;

(** Prints the SOAP object into a string *)
value print_value : ty -> string ;
